import requests
from django.db import models
from django.utils.translation import gettext, gettext_lazy as _


# Create your models here.


class Region(models.Model):

    class Meta:
        verbose_name = _('Region')
        verbose_name_plural = _('Regions')

    name = models.CharField(
        verbose_name=_('Name'),
        max_length=254
    )

    acronym = models.CharField(
        verbose_name=_('Acronym'),
        max_length=2
    )

    def __str__(self):
        return self.name


class State(models.Model):

    class Meta:
        verbose_name = _('State')
        verbose_name_plural = _('States')

    name = models.CharField(
        verbose_name=_('Name'),
        max_length=254
    )

    acronym = models.CharField(
        verbose_name=_('Acronym'),
        max_length=2
    )

    region = models.ForeignKey(
        'Region',
        verbose_name=_('Region'),
        on_delete=models.CASCADE
    )

    def __str__(self):
        return self.name


class City(models.Model):

    class Meta:
        verbose_name = _('City')
        verbose_name_plural = _('Cities')

    name = models.CharField(
        verbose_name=_('Name'),
        max_length=254
    )

    state = models.ForeignKey(
        'State',
        verbose_name=_('State'),
        on_delete=models.CASCADE
    )

    def __str__(self):
        return self.name


class Address(models.Model):

    class Meta:
        verbose_name = _('Address')
        verbose_name_plural = _('Addresses')

    zipcode = models.CharField(
        verbose_name=_('Zipcode'),
        max_length=8,
        blank=True,
        null=True
    )

    state = models.ForeignKey(
        'State',
        verbose_name=_('State'),
        on_delete=models.CASCADE,
        blank=True,
        null=True
    )

    city = models.ForeignKey(
        'City',
        verbose_name=_('City'),
        on_delete=models.CASCADE,
        blank=True,
        null=True
    )

    district = models.CharField(
        verbose_name=_('District'),
        max_length=254
    )

    address = models.CharField(
        verbose_name=_('Address'),
        max_length=254
    )

    number = models.IntegerField(
        verbose_name=_('Number'),
    )

    complement = models.CharField(
        verbose_name=_('Complement'),
        max_length=254
    )

    def __str__(self):
        return self.address


def import_regions():
    """
    docstring
    """

    response = requests.get('https://servicodados.ibge.gov.br/api/v1/localidades/regioes')
    data = response.json()

    for item in data:
        region = Region()
        region.pk = item.get("id")
        region.name = item.get("nome")
        region.acronym = item.get("sigla")
        region.save()


def import_state():
    """
    docstring
    """

    response = requests.get('https://servicodados.ibge.gov.br/api/v1/localidades/estados')
    data = response.json()

    for item in data:
        state = State()
        state.pk = item.get("id")
        state.name = item.get("nome")
        state.acronym = item.get("sigla")
        state.region = Region.objects.get(pk=item.get("regiao").get("id"))
        state.save()


def import_cities():
    """
    docstring
    """

    response = requests.get('https://servicodados.ibge.gov.br/api/v1/localidades/municipios')
    data = response.json()

    for item in data:
        city = City()
        city.pk = item.get("id")
        city.name = item.get("nome")
        city.state = State.objects.get(pk=item.get("microrregiao").get("mesorregiao").get("UF").get("id"))
        city.save()



# Generated by Django 3.1.7 on 2021-02-25 23:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Region',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=254, verbose_name='Nome')),
                ('acronym', models.CharField(max_length=2, verbose_name='Sigla')),
            ],
            options={
                'verbose_name': 'Região',
                'verbose_name_plural': 'Regiões',
            },
        ),
        migrations.CreateModel(
            name='State',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=254, verbose_name='Nome')),
                ('acronym', models.CharField(max_length=2, verbose_name='Sigla')),
                ('region', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangoaddressesapp.region', verbose_name='Região')),
            ],
            options={
                'verbose_name': 'Estado',
                'verbose_name_plural': 'Estados',
            },
        ),
        migrations.CreateModel(
            name='City',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=254, verbose_name='Nome')),
                ('state', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangoaddressesapp.state', verbose_name='')),
            ],
            options={
                'verbose_name': 'Município',
                'verbose_name_plural': 'Municípios',
            },
        ),
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('complement', models.CharField(blank=True, max_length=254, null=True, verbose_name='Complemento')),
                ('zipcode', models.CharField(blank=True, max_length=8, null=True, verbose_name='CEP')),
                ('district', models.CharField(max_length=254, verbose_name='Bairro')),
                ('address', models.CharField(max_length=254, verbose_name='Endereço')),
                ('number', models.IntegerField(verbose_name='Número')),
                ('city', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoaddressesapp.city', verbose_name='Município')),
                ('state', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoaddressesapp.state', verbose_name='Estado')),
            ],
            options={
                'verbose_name': 'Endereço',
                'verbose_name_plural': 'Endereços',
            },
        ),
    ]



import requests
from django.core.management.base import BaseCommand, CommandError
from djangoaddressesapp import import_regions, import_state, import_cities

class Command(BaseCommand):
    help = 'Import regions, states and cities by IBGE api'

    def handle(self, *args, **options):
        
        # Import Regions
        self.stdout.write(self.style.NOTICE('Initializing Import of Regions by IBGE API.'))
        import_regions()
        self.stdout.write(self.style.SUCCESS('Finalized import of Regions.'))

        # Import States
        self.stdout.write(self.style.NOTICE('Initializing Import of States by IBGE API.'))
        import_state()
        self.stdout.write(self.style.SUCCESS('Finalized import of States.'))

        # Import Cities
        self.stdout.write(self.style.NOTICE('Initializing Import of Cities by IBGE API.'))
        import_cities()
        self.stdout.write(self.style.SUCCESS('Finalized import of Cities.'))



from django.contrib import admin
from .models import Region, State, City, Address

# Register your models here.

@admin.register(Region)
class RegionAdmin(admin.ModelAdmin):

    list_display = [
        "name",
        "acronym",
    ]

    list_display_links = [
        "name",
        "acronym",
    ]
    
    search_fields = [
        "name",
        "acronym",
    ]

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, *args):
        return False

    def has_delete_permission(self, request, *args):
        return False


@admin.register(State)
class StateAdmin(admin.ModelAdmin):

    list_display = [
        "name",
        "acronym",
        "region",
    ]

    list_display_links = [
        "name",
        "acronym",
        "region",
    ]

    list_filter = [
        "region",
    ]
    
    search_fields = [
        "name",
        "acronym",
    ]

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, *args):
        return False

    def has_delete_permission(self, request, *args):
        return False


@admin.register(City)
class CityAdmin(admin.ModelAdmin):

    list_display = [
        "name",
        "state",
    ]

    list_display_links = [
        "name",
        "state",
    ]
    
    list_filter = [
        "state",
        "state__region",
    ]

    search_fields = [
        "name",
        "state__name",
    ]

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, *args):
        return False

    def has_delete_permission(self, request, *args):
        return False


@admin.register(Address)
class AddressAdmin(admin.ModelAdmin):

    list_display = [
        "zipcode",
        "state",
        "city",
        "district",
        "address",
        "number",
    ]

    list_display_links = [
        "zipcode",
        "state",
        "city",
        "district",
        "address",
        "number",
    ]
    
    search_fields = [
        "zipcode",
        "state__name",
        "city__name",
        "district",
        "address",
        "number",
    ]

    autocomplete_fields = [
        "state",
        "city",
    ]
